/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
//
// PBC

#ifndef __pbc_application_h__
#define __pbc_application_h__

#include <map>
#include "ns3/address.h"
#include "ns3/application.h"
#include "ns3/event-id.h"
#include "ns3/ptr.h"
#include "ns3/data-rate.h"
#include "ns3/random-variable.h"
#include "ns3/traced-callback.h"
#include "ns3/header.h"
#include "ns3/tag.h"
#include "ns3/buffer.h"
#include "ns3/vector.h"
#include "ns3/double.h"
#include <ns3/mobility-model.h>
#include <ns3/intelligent-driver-mobility-model.h>
#include "ns3/log.h"
#include "ns3/wifi-phy.h"



namespace ns3 {

class Address;
class RandomVariable;
class Socket;


/**
 * \ingroup applications
 * \defgroup pbc PeriodicBroadcastApplication
 *
 * This traffic generator follows the periodic broadcast agent style as it is
 * envisioned in vehicular networks.
 *
 * Once the application has been started, it sends packets at a fixed rate
 * , e.g. at 5 Hz
 *
 * The payload of the packets can either be set manually using the Attribute
 * "Payload" or left empty. In the latter case (default), no payload will be used.
 */
class PbcApplication : public Application
{
public:
  static TypeId GetTypeId (void);

  PbcApplication ();
  virtual ~PbcApplication();
  
  
protected:

  virtual void DoDispose (void);
  
private:

    

    // inherited from Application base class.
  virtual void StartApplication (void);    // Called at time specified by Start
  virtual void StopApplication (void);     // Called at time specified by Stop

    // helpers
  void CancelEvents ();

    // Event handlers
  void ReceivePacket (Ptr<Socket> socket);
  void SendPacket();

  void PhyStateTrace (std::string context, Time start, Time duration, enum WifiPhy::State state);
  void UpdateCBR (void);
  
  Ptr<Socket>   m_socket;     // associated socket
  Address       m_peer;       // peer address
  Time          m_interval;   // the time interval
  uint32_t	    m_nodeId;       // nodeID
  TypeId        m_protocol;
  uint32_t	    m_packetSize;
  bool          m_pcbract;
  bool          m_recvact;
  int           m_sendcount;
  double        m_jitter;

  EventId       m_sendEvent;  // event id of next "send packet" event
  EventId       m_cbrEvent; 

  Vector        m_pos;        // current position

  Time          m_busyTimes;
  Time          m_idleTimes;
  
  TracedCallback<double, double> m_cbrTrace;
  TracedCallback<> m_sendTrace;
  TracedCallback<> m_recvTrace;
};

} // namespace ns3

#endif

