/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
//
// Traffic Light

#ifndef __traffic_light_h__
#define __traffic_light_h__

#include <map>
#include "ns3/address.h"
#include "ns3/application.h"
#include "ns3/event-id.h"
#include "ns3/ptr.h"
#include "ns3/data-rate.h"
#include "ns3/random-variable.h"
#include "ns3/traced-callback.h"
#include "ns3/header.h"
#include "ns3/tag.h"
#include "ns3/buffer.h"
#include "ns3/vector.h"
#include "ns3/double.h"
#include <ns3/mobility-model.h>
#include <ns3/intelligent-driver-mobility-model.h>
#include "ns3/log.h"
#include "vtl-client.h"


namespace ns3 {

class Address;
class RandomVariable;
class Socket;




class TrafficLight : public Application
{
public:
  static TypeId GetTypeId (void);

  TrafficLight ();
  virtual ~TrafficLight();
  
  typedef struct VtlHeader::VtlInfo VtlInfo;

  Ptr<Intersection>                     m_int;

protected:

  virtual void DoDispose (void);
  
  
private:

    // inherited from Application base class.
  virtual void StartApplication (void);    // Called at time specified by Start
  virtual void StopApplication (void);     // Called at time specified by Stop

    // helpers
  void CancelEvents ();
  void InitVtl ();
  
    // Event handlers
  void NextTimestep();
  void SendPacket();
  void ScheduleNextTx();
  void VtlExpire (void);
  void LeaderUpdateTimeslice (void);
  
  
  
  // constant/configuration
  Ptr<Socket>   m_socket;     // associated socket
  Address       m_peer;       // peer address
  Time          m_interval;   // the time interval
  TypeId        m_protocol;   // the underlying protocol which is used
  uint32_t      m_seqNo;      // sequence number counter
  uint32_t      m_id;         // nodeID
  bool          m_sync;

  uint32_t                              m_intersectionId;
  
  std::list<VtlInfo>                    m_vtlList;    
  Time                                  m_phaseLength;
  uint32_t                              m_currentPhase;
  std::vector<std::list<Ptr<Road> > >   m_roadmap;
  
  
  // helper, events etc.
  EventId       m_sendEvent;
  EventId       m_leaderUpdateEvent; 
  
  
  
};

} // namespace ns3

#endif

