/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
//

#ifndef __vtl_calc_h__
#define __vtl_calc_h__


#include "ns3/vector.h"
#include <math.h>
#include "ns3/object.h"
#include "ns3/manhattan-idm.h"

namespace ns3 {
  
class VtlCalc : public Object
{
public:
  struct params {
    double txPowerDbm;
    double rxThres;
    double roadsize;    // inter building distance for nlos model
    double stopdist;    // distance intersection center to stopped vehicle
    double offset;      // distance road middle to vehicle
    bool suburban;
    double maxdist;     // maximum observation distance (e.g. road length)
    double prob;
    double pgr;
    double maxspeed;
    double collprob;
    
    double systemloss;
    double stddev;
    double lambda;
    double height;
  } p;




  VtlCalc ();
  ~VtlCalc ();
  
  
  double    IsRoadEmpty(double curDist, double maxTime, Road::roadrelation o);
  

  static Ptr<VtlCalc> GetPtr();
  
private:
  static Ptr<VtlCalc>   m_vtlcalcobj;

  double    GetSingleRecieveProb(double dist, Road::roadrelation o);
  double    calcRxProb (double rxPwr);
  Vector    getSenderPosition (double dist);
  Vector    getReceiverPosition (double dist, Road::roadrelation o);
  double    calcRxPwr(const Vector &srcPos, const Vector &dstPos);
  double    getPr0 (double dr, double dt, double xt, double roadsize, bool isSuburban) const;
  double    Phi (double x, double stddev);
  Vector    vdiff (const Vector &v, const Vector &w) const;


};

} // ns3
#endif
