#ifndef MANHATTAN_TOPOLOGY_HELPER_H
#define MANHATTAN_TOPOLOGY_HELPER_H

#include <vector>
#include <list>
#include "ns3/object-factory.h"
#include "ns3/attribute.h"
#include "ns3/position-allocator.h"
#include "ns3/node-container.h"
#include "ns3/random-variable.h"


#include <ns3/intelligent-driver-mobility-model.h>
#include <ns3/manhattan-idm.h>

namespace ns3 {

class ManhattanTopologyHelper
{

public:
  ManhattanTopologyHelper ();
  ~ManhattanTopologyHelper ();


  void     makeSimpleTopo (int blocksV, int blocksH, double blockSizeV, 
      double blockSizeH, double ibd, bool suburban, Time endTime);
      
  void    makeVehicleSources (Ptr<Road> source, Time endTime);
  void    makeSinglePath (int starti, int startj, int endi, int endj,
    Vector direction, bool);
  void    connectRoad (Ptr<Road> road, Ptr<Intersection> from, 
      Ptr<Intersection> to, Vector direction);
  std::string     getIntersectionLayout(void);
  
  RandomVariable speedvar;
  RandomVariable thvar;
  RandomVariable accvar;
  RandomVariable decvar;
  RandomVariable ivtvar;
  RandomVariable ivtmainvar;
  
  NodeContainer  m_nc;  
  Callback<void,Ptr<Node>,double> m_callback;
  
  int     m_mainDir;
  double  m_laneWidth;
  double  m_sourceLength;
  bool    m_setNoConflict;
  bool    m_makeTrafficLights;
  bool    m_stlsync;
  
private:
  void    makeTrafficLights (Ptr<Intersection> curint, Time endTime);
  
  
  int m_roadC;
  std::list<Ptr<Road> > m_sourceRoads;
  
  std::vector<Ptr<Intersection> > m_intersections;
  
  int m_blocksH;
  int m_blocksV;
  double m_blockSizeV;
  double m_blockSizeH;
  
  Ptr<Intersection> iget(int i, int j);
  void iset(int i, int j, Ptr<Intersection> intersection);
};



}

#endif
