/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */




#ifndef VIRTUALSOURCE11P_PROPAGATION_LOSS_MODEL_H
#define VIRTUALSOURCE11P_PROPAGATION_LOSS_MODEL_H

#include "ns3/random-variable.h"
#include "propagation-loss-model.h"
#include "ns3/vector.h"
#include <cmath>
#include "ns3/random-variable-stream.h"
#include "ns3/manhattan-idm.h"

namespace ns3 {

/**
 * \ingroup propagation
 *
 * \brief NLOS propagation model for (urban) crossings
 *
 * See "A Validated 5.9 Ghz Non-Line-Of-Sight Path-Loss and Fading Model..."
 * by T. Mangel et al
 *
 * Crossing is always symmetric 4 way, 90° crossing.
 */ 
class VirtualSource11pPropagationLossModel : public PropagationLossModel
{
public:
  static TypeId GetTypeId (void);

  VirtualSource11pPropagationLossModel ();
  virtual ~VirtualSource11pPropagationLossModel ();
  /**
   * \param rs (m) The road's size
   *
   * Set the roadsize
   */
  void SetRoadSize (double rs);

private:
  VirtualSource11pPropagationLossModel (const VirtualSource11pPropagationLossModel &o);
  VirtualSource11pPropagationLossModel & operator = (const VirtualSource11pPropagationLossModel &o);
  virtual double DoCalcRxPower (double txPowerDbm,
                                Ptr<MobilityModel> a,
                                Ptr<MobilityModel> b) const;
  virtual double DoCalcRxPowerNoFad (double txPowerDbm,
                                Ptr<MobilityModel> a,
                                Ptr<MobilityModel> b) const;
  double  DoCalcRxPower (double txPowerDbm,
                                Ptr<MobilityModel> a,
                                Ptr<MobilityModel> b,
                                bool withfading) const;
  double  NakagamiCalcRxPower (double txPowerDbm,
                                Ptr<MobilityModel> a,
                                Ptr<MobilityModel> b) const;
  double  getPr0 (double dr, double dt, double xt, double roadsize, bool isSuburban) const;
  bool    GetCommonIntersection (Ptr<MobilityModel> a, Ptr<MobilityModel> b, 
                                  Vector &cs, double &rs, bool &suburban) const;

  Vector vplus (const Vector v, const Vector w) const;
  Vector vdiff (const Vector v, const Vector w) const;
  
  virtual int64_t DoAssignStreams (int64_t stream);
    
  double        m_roadsize;
  double        m_lambda;
  double        m_systemLoss;
  double        m_heightAboveZ;
  Vector        m_crossingCenter;
  bool          m_isSuburban;
  
  Ptr<NormalRandomVariable>        m_normVar;
  Ptr<ErlangRandomVariable>        m_erlangRandomVariable;
  Ptr<GammaRandomVariable>         m_gammaRandomVariable;
  
  Ptr<ThreeLogDistancePropagationLossModel>      m_threeLogDistPLM;
  
};

} // namespace ns3

#endif /* VIRTUALSOURCE11P_PROPAGATION_LOSS_MODEL_H */

